﻿using Hotline.CallCenter.Calls;
using Hotline.Orders;
using Hotline.Permissions;
using Hotline.Settings;
using Hotline.Settings.CommonOpinions;
using Hotline.Share.Dtos.Settings;
using Hotline.Share.Enums.FlowEngine;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using MongoDB.Driver;
using SqlSugar;
using System.Reflection.Metadata;
using Hotline.FlowEngine.Workflows;
using Hotline.Settings.TimeLimits;
using Hotline.Share.Enums.Order;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Api.Controllers
{
    /// <summary>
    /// 常用意见接口
    /// </summary>
    public class CommonPController: BaseController
    {
        private readonly ISystemCommonOpinionDomainService _commonOpinionDomainService;
        private readonly ISystemAreaDomainService _systemAreaDomainService;
        private readonly IMapper _mapper;
        private readonly ISessionContext _sessionContext;
        private readonly IRepository<TrCallRecord> _trCallRecordRepository;
        private readonly IOrderRepository _orderRepository;
        private readonly IOrderDelayRepository _orderDelayRepository;
        private readonly ITimeLimitDomainService _timeLimitDomainService;

		public CommonPController(
            ISystemCommonOpinionDomainService commonOpinionDomainService,
            ISystemAreaDomainService systemAreaDomainService,
            ISessionContext sessionContext,
            IRepository<TrCallRecord> trCallRecordRepository,
            IOrderRepository orderRepository,
			IMapper mapper,
            IOrderDelayRepository orderDelayRepository,
            ITimeLimitDomainService timeLimitDomainService)
        {
            _commonOpinionDomainService = commonOpinionDomainService;
            _systemAreaDomainService= systemAreaDomainService;
            _mapper = mapper;
            _sessionContext = sessionContext;
            _trCallRecordRepository = trCallRecordRepository;
            _orderRepository = orderRepository;
            _orderDelayRepository = orderDelayRepository;
            _timeLimitDomainService = timeLimitDomainService;
		}

       

        #region 省市区
        /// <summary>
        /// 获取省市区树形
        /// </summary>
        /// <returns></returns>
        [HttpGet("tree-area")]
        [Obsolete("请调用sys/area/tree")]
        public async Task<List<SystemArea>> GetAearTree()
        {
            return await _systemAreaDomainService.GetAreaTree();
        }

        #endregion

        /// <summary>
        /// 首页基础数据
        /// </summary>
        /// <returns></returns>
        [HttpGet("home_data")]
        public async Task<Object> GetHomeData() 
        {
            var tadayTime = DateTime.Now.ToString("yyyy-MM-dd");
			//中心
			if (_sessionContext.OrgIsCenter)
	        {
		        var orderQuery = _orderRepository.Queryable(false, false, false)
			        .Includes(o => o.Workflow, w => w.Steps);
				//今日来电
				var tadayCalls = await _trCallRecordRepository.Queryable()
	                .Where(x => x.CallDirection == Share.Enums.CallCenter.ECallDirection.In && tadayTime.Equals(x.CreatedTime.ToString("yyyy-MM-dd"))).ToListAsync();
				var callNum = tadayCalls.Count();
                var validCallNum = tadayCalls.Where(x => x.Duration > 0 || x.QueueTims > 0 || x.RingTimes > 0).Count();
				//今日接通率
                var answeredNum = tadayCalls.Where(x=>x.Duration > 0).Count();
                var answeredRate = validCallNum > 0 ? (answeredNum / validCallNum * 100) + "%" : "100%";
                //今日受理工单
                var tadayOrders = await orderQuery
	                .Where(o => tadayTime.Equals(o.CreationTime.ToString("yyyy-MM-dd"))).ToListAsync();
				var orderNum = tadayOrders.Count();
				var directlyNum = tadayOrders.Where(o=> o.ProcessType == Share.Enums.Order.EProcessType.Zhiban).Count();
                return new { CallNum = callNum, ValidCallNum = validCallNum, AnsweredNum = answeredNum, AnsweredRate = answeredRate, OrderNum = orderNum, DirectlyNum = directlyNum };
			}
			//部门
			//今日待办 tasksOkNum
			var time = DateTime.Parse(tadayTime);
            var order = await _orderRepository.Queryable()
               .Includes(o => o.Workflow, w => w.Steps)
               .Where(o => SqlFunc.JsonListObjectAny(o.Workflow.HandlerUsers, "Key", _sessionContext.RequiredUserId) || SqlFunc.JsonListObjectAny(o.Workflow.HandlerOrgs, "Key", _sessionContext.RequiredOrgId))
               .GroupBy(o => o.Id)
               .Select(o => new
               {
                   aboutExpire = SqlFunc.AggregateSum(SqlFunc.IIF(time > o.NearlyExpiredTime!.Value && time < o.ExpiredTime!.Value, 1, 0)),
                   havExpired = SqlFunc.AggregateSum(SqlFunc.IIF( time > o.ExpiredTime!.Value, 1, 0)),
                   countersignHandle = SqlFunc.AggregateSum(SqlFunc.IIF(o.CounterSignType.HasValue, 1, 0)),

			   }).FirstAsync();
            var aboutExpire = order?.aboutExpire ?? 0;
            var havExpired = order?.havExpired ?? 0;
            var countersignHandle = order?.countersignHandle ?? 0;



			return new { AboutExpire = aboutExpire, HavExpired = havExpired, CountersignHandle = countersignHandle, ScreenAudit = "aa", Delay = "aa", ScreenHandle = "aa" };
        }
	}
}
