﻿using Hotline.Caching.Interfaces;
using Hotline.CallCenter.Calls;
using Hotline.Orders;
using Hotline.Realtimes;
using Hotline.Share.Dtos.TrCallCenter;
using MapsterMapper;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using NewRock.Sdk.Events;
using Org.BouncyCastle.Utilities;
using Serilog.Core;
using SqlSugar;
using StackExchange.Redis;
using System;
using System.Collections.Generic;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Tr.Sdk;
using Tr.Sdk.Tels;
using XF.Domain.Constants;
using XF.Domain.Repository;

namespace Hotline.Application.Bigscreen
{
	public class SeatStateDataRefreshService : BackgroundService
	{
		private readonly IServiceScopeFactory _serviceScopeFactory;
		private DateTime _lastModifyTime;

		public SeatStateDataRefreshService(IServiceScopeFactory serviceScopeFactory)
		{
			_serviceScopeFactory = serviceScopeFactory;
			_lastModifyTime = DateTime.Now;
		}

		protected override async Task ExecuteAsync(CancellationToken stoppingToken)
		{
			using var scope = _serviceScopeFactory.CreateScope();
			var realtimeService = scope.ServiceProvider.GetRequiredService<IRealtimeService>();
			var logger = scope.ServiceProvider.GetRequiredService<ILogger<SeatStateDataRefreshService>>();
			var systemSettingCacheManager = scope.ServiceProvider.GetRequiredService<ISystemSettingCacheManager>();
			var seatStateDataService = scope.ServiceProvider.GetRequiredService<ISeatStateDataService>();

			try
			{
				int times = int.Parse(systemSettingCacheManager.GetSetting(SettingConstants.BsDataStateChangedTimes)?.SettingValue[0]);

				while (!stoppingToken.IsCancellationRequested)
				{
					await realtimeService.BsSeatStateDataShowChangedAsync1(await seatStateDataService.GetCall24(stoppingToken), stoppingToken);
					await realtimeService.BsSeatStateDataShowChangedAsync2(await seatStateDataService.GetCallTop10(stoppingToken), stoppingToken);
					await realtimeService.BsSeatStateDataShowChangedAsync3(await seatStateDataService.GetCallList(stoppingToken), stoppingToken);
					await realtimeService.BsSeatStateDataShowChangedAsync4(await seatStateDataService.GetCallAverage(stoppingToken), stoppingToken);
					//logger.LogInformation($"send msg: {orders.First().Address}");
					_lastModifyTime = DateTime.Now;
					//}
					await Task.Delay(times, stoppingToken);
				}
			}
			catch (Exception e)
			{
				logger.LogInformation($"send msg: {e.ToString()}");
				throw;
			}
		}
	}
}
