﻿using DotNetCore.CAP;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.FlowEngine.Workflows;
using Hotline.Orders;
using Hotline.Settings.TimeLimits;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;
using MapsterMapper;
using MediatR;
using Microsoft.Extensions.Logging;

namespace Hotline.Application.Handlers.FlowEngine;

public class WorkflowRecallHandler : INotificationHandler<RecallNotify>
{
    private readonly IOrderDomainService _orderDomainService;
    private readonly IOrderRepository _orderRepository;
    private readonly IWorkflowDomainService _workflowDomainService;
    private readonly ITimeLimitDomainService _timeLimitDomainService;
    private readonly ICapPublisher _capPublisher;
    private readonly IMapper _mapper;
    private readonly ILogger<WorkflowRecallHandler> _logger;

    public WorkflowRecallHandler(
        IOrderDomainService orderDomainService,
        IOrderRepository orderRepository,
        IWorkflowDomainService workflowDomainService,
        ITimeLimitDomainService timeLimitDomainService,
        ICapPublisher capPublisher,
        IMapper mapper,
        ILogger<WorkflowRecallHandler> logger)
    {
        _orderDomainService = orderDomainService;
        _orderRepository = orderRepository;
        _workflowDomainService = workflowDomainService;
        _timeLimitDomainService = timeLimitDomainService;
        _capPublisher = capPublisher;
        _mapper = mapper;
        _logger = logger;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(RecallNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;
        var data = notification.Dto;

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.OrderHandle:
                var order = await _orderDomainService.GetOrderAsync(workflow.ExternalId, withHotspot: true, withAcceptor: true, 
                    withExtension: true, cancellationToken: cancellationToken);
                _mapper.Map(workflow, order);
                if (notification.TargetStep.StepType is EStepType.Start)
                {
                    order.Status = EOrderStatus.SpecialToUnAccept;
                    order.BackToUnsign();
                }

                await _orderRepository.UpdateAsync(order, cancellationToken);

                //if (data.External != null && data.External.TimeLimit.HasValue && data.External.TimeLimitUnit.HasValue)
                //{
                //    var expiredTime = _timeLimitDomainService.CalcEndTime(DateTime.Now,
                //        data.External.TimeLimitUnit.Value,
                //        data.External.TimeLimit.Value, order.AcceptTypeCode);

                //    await _workflowDomainService.UpdateExpiredTimeAsync(workflow, expiredTime.EndTime,
                //        expiredTime.RuleStr, data.External.TimeLimit, data.External.TimeLimitUnit, expiredTime.NearlyExpiredTime, cancellationToken);

                //    var dto = _mapper.Map<OrderDto>(order);
                //    await _capPublisher.PublishAsync(Hotline.Share.Mq.EventNames.HotlineOrderExpiredTimeUpdate, dto, cancellationToken: cancellationToken);
                //}

                break;
            case WorkflowModuleConsts.KnowledgeAdd:
            case WorkflowModuleConsts.KnowledgeUpdate:
            case WorkflowModuleConsts.KnowledgeDelete:
            case WorkflowModuleConsts.TelRestApply:
                break;
        }
    }
}