﻿using Hotline.FlowEngine.Workflows;
using Hotline.Repository.SqlSugar.DataPermissions;
using SqlSugar;
using XF.Domain.Entities;

namespace Hotline.Repository.SqlSugar.Extensions
{
    public static class DataPermissionExtensions
    {
        public static ISugarQueryable<TEntity> DataPermissionFiltering<TEntity>(this ISugarQueryable<TEntity> queryable,
            IDataPermissionFilterBuilder dataPermissionFilterBuilder)
            where TEntity : class, IEntity<string>, IDataPermission, new()
        {
            return queryable.Where(dataPermissionFilterBuilder.Build<TEntity>());
        }

        public static ISugarQueryable<TEntity> WorkflowViewFiltering<TEntity>(this ISugarQueryable<TEntity> queryable,
            IDataPermissionFilterBuilder dataPermissionFilterBuilder)
            where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
        {
            return queryable.Where(dataPermissionFilterBuilder.BuildWithFlowViewFilter<TEntity>());
        }

        public static ISugarQueryable<TEntity> WorkflowHandleFiltering<TEntity>(this ISugarQueryable<TEntity> queryable,
            IDataPermissionFilterBuilder dataPermissionFilterBuilder, bool canHandle)
            where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
        {
            return queryable.Where(dataPermissionFilterBuilder.BuildWithFlowHandleFilter<TEntity>(canHandle));
        }

        public static TEntity InitDatePermission<TEntity>(this TEntity entity,
            IDataPermissionManager dataPermissionManager)
            where TEntity : class, IEntity<string>, IDataPermission, new()
        {
            var result = dataPermissionManager.GetDataPermissionOptions();
            entity.CreateDataPermission(result.creatorId, result.creatorName, result.orgId, result.orgName,
                result.orgLevel, result.areaId);
            return entity;
        }

        public static TEntity AssignToCreator<TEntity>(this TEntity entity,
            IDataPermissionManager dataPermissionManager)
            where TEntity : class, IEntity<string>, IDataPermission, IWorkflow, new()
        {
            var (creatorId, _, _, _, _, _) = dataPermissionManager.GetDataPermissionOptions();
            entity.Assign(EFlowAssignType.User, creatorId);
            return entity;
        }
    }
}