﻿using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Ai;
using Hotline.Share.Requests;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Ai
{
    public class AiVisitBackDto
    {
        public string BatchUid { get; set; }

        public string BatchName { get; set; }

        public string SceneUid { get; set; }

        public string SenceName { get; set; }

        public string TaskUid { get; set; }

        public string CalledNumber { get; set; }
        /// <summary>
        /// 任务状态( 1 :待执行、2 :暂停中、3 :执行中、4 :失效、5 :呼叫失败 、6：已结束（完成或取消）)
        /// </summary>
        public int TaskStatus { get; set; }

        public int CallTimes { get; set; }

        public List<CallRecordDto> CallRecordList { get; set; }
    }

    public class CallRecordDto
    {
        public int CallNo { get; set; }

        /// <summary>
        /// 呼叫状态(1:呼叫中、2:完成、3:呼叫失败)
        /// </summary>
        public int CallStatus { get; set; }

        /// <summary>
        /// 回访结果 (1成功、2:不涉及、3:失败)
        /// </summary>
        public int ReturnVisit { get; set; }

        public List<QuestionnaireResult> QuestionnaireResult { get; set; }

        /// <summary>
        /// 不满意原因
        /// </summary>
        public Dictionary<string, string> SceneVariable { get; set; }
    }

    public class QuestionnaireResult
    {
        public string QuestionId { get; set; }

        public string QuestionChoice { get; set; }

        public string QuestionResult { get; set; }
    }


    public record AiVisitListDto:PagedKeywordRequest
    {
        
    }

    public record AiVisitDetailListDto:PagedKeywordRequest
    {
        public string Id { get; set; }
    }

    public class AiOrderVisitDto
    {
        public string Id { get; set; }
        public string Name { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        public EAiOrderVisitTaskState TaskState { get; set; }

        public string TaskStateText => TaskState.GetDescription();

        /// <summary>
        /// 批次上报结果ID
        /// </summary>
        public string? BatchUid { get; set; }


        /// <summary>
        /// 节日禁呼 0：否 1：是
        /// </summary>
        public int FestivalBan { get; set; }

        /// <summary>
        /// 外呼规则类型 1：自定义 2：：使用现有规则
        /// </summary>
        public int RuleType { get; set; }

        /// <summary>
        /// 现有外呼规则ID
        /// </summary>
        public string RuleUid { get; set; }

        /// <summary>
        /// 应回访数量
        /// </summary>
        public int HasVisitCount { get; set; }

        /// <summary>
        /// 已回访成功数量
        /// </summary>
        public int VisitedCount { get; set; }

        /// <summary>
        /// 已回访失败数量
        /// </summary>
        public int VisitedFailCount { get; set; }

        public DateTime CreationTime { get; set; }

        public string? CreatorName { get; set; }
    }

    public class AiOrderVisitDetailDto
    {
        /// <summary>
        /// 主表ID
        /// </summary>
        public string AiOrderVisitId { get; set; }
        /// <summary>
        /// 工单ID
        /// </summary>
        public string OrderId { get; set; }

        /// <summary>
        /// 工单
        /// </summary>
        public OrderDto Order { get; set; }
        /// <summary>
        /// 工单回访主表ID
        /// </summary>
        public string OrderVisitId { get; set; }
        /// <summary>
        /// 工单
        /// </summary>
        public OrderVisitDto OrderVisit { get; set; }
        /// <summary>
        /// 外呼电话
        /// </summary>
        public string OuterNo { get; set; }

        public EAiOrderVisitState AiOrderVisitState { get; set; }

        public string AiOrderVisitStateText => AiOrderVisitState.GetDescription();

        /// <summary>
        /// 批此上传成功后任务ID
        /// </summary>
        public string? TaskUid { get; set; }
    }

    public class AddAiVisitDto
    {
        public string Name { get; set; }

        public DateTime BeginTime { get; set; }

        public DateTime EndTime { get; set; }

        /// <summary>
        /// 节日禁呼 0：否 1：是
        /// </summary>
        public int FestivalBan { get; set; }

        public List<AddAiOrderVisitDetailDto> AiOrderVisitDetails { get; set; }
    }

    public class AddAiOrderVisitDetailDto
    {
        /// <summary>
        /// 工单ID
        /// </summary>
        public string OrderId { get; set; }
        /// <summary>
        /// 工单回访主表ID
        /// </summary>
        public string OrderVisitId { get; set; }

        /// <summary>
        /// 外呼电话
        /// </summary>
        public string OuterNo { get; set; }
    }
}
