﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Hotline.Share.Dtos.File;
using Hotline.Share.Dtos.Users;
using Hotline.Share.Enums.Order;
using Hotline.Share.Enums.Settings;
using Hotline.Share.Requests;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.Order
{
    public record QueryOrderDto : PagedKeywordRequest
    {
        /// <summary>
        /// 省本地编号（√）
        /// </summary>
        public string ProvinceNo { get; set; }

        /// <summary>
        /// 本地编号（√）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 诉求内容
        /// </summary>
        //public string? Content { get; set; }

        /// <summary>
        /// 受理类型（√）
        /// </summary>
        public List<string> AcceptTypes { get; set; } = new();

        /// <summary>
        /// 来源渠道（√）
        /// </summary>
        public List<string> Channels { get; set; } = new();

        /// <summary>
        /// 转接号码（转接来源）（√）
        /// </summary>
        public string? TransferPhone { get; set; }

        /// <summary>
        /// 热点分类（√）
        /// </summary>
        public List<string> HotspotIds { get; set; } = new();

        /// <summary>
        /// 接办部门（√）
        /// </summary>
        public List<string> OrgCodes { get; set; } = new();

        /// <summary>
        /// 受理坐席名字或工号（×）
        /// </summary>
        public string? NameOrNo { get; set; }

        /// <summary>
        /// 受理时间(工单创建时间)（√）
        /// </summary>
        public DateTime? CreationTimeStart { get; set; }
        public DateTime? CreationTimeEnd { get; set; }

        /// <summary>
        /// 紧急程度（√）
        /// </summary>
        public List<EEmergencyLevel> EmergencyLevels { get; set; } = new();

        /// <summary>
        /// 来电号码（×）
        /// </summary>
        public string? FromPhone { get; set; }


        /// <summary>
        /// 来电或联系电话（√）
        /// </summary>
        public string? PhoneNo { get; set; }

        /// <summary>
        /// 推送分类（√）(单选)
        /// </summary>
        public string? PushTypeCode { get; set; }

        /// <summary>
        /// 期满时间（√）
        /// </summary>
        public DateTime? ExpiredTimeStart { get; set; }
        public DateTime? ExpiredTimeEnd { get; set; }


        /// <summary>
        /// 工单状态（√）
        /// </summary>
        public List<EOrderStatus> Statuses { get; set; } = new();

        /// <summary>
        /// 接办人（√）
        /// </summary>
        public string? ActualHandlerName { get; set; }

        /// <summary>
        /// 是否甄别（√）
        /// </summary>
        public bool? IsScreen { get; set; }

        /// <summary>
        /// 当前办理节点（√）（单选）
        /// </summary>
        public string CurrentStepCode { get; set; }
        /// <summary>
        /// 办结时间（√）
        /// </summary>
        public DateTime? ActualHandleTimeStart { get; set; }
        public DateTime? ActualHandleTimeEnd { get; set; }

        /// <summary>
        /// 是否超期（√）
        /// </summary>
        public bool? IsOverTime { get; set; }

        /// <summary>
        /// 来电主体（√）(单选)
        /// </summary>
        public EIdentityType? IdentityType { get; set; }

        /// <summary>
        /// 来电人姓名（×）
        /// </summary>
        public string? FromName { get; set; }

        /// <summary>
        /// 区域（×）(多选)
        /// </summary>
        public List<string> AreaCodes { get; set; } = new();

        /// <summary>
        /// 是否省工单（空为全部  true为省工单 false为市工单）
        /// </summary>
        public bool? IsProvinceOrder { get; set; }
    }


    public record QueryOrderPublishDto : PagedKeywordRequest
    {
        /// <summary>
        /// 发布状态
        /// </summary>
        public EPubState? PubState { get; set; }


        /// <summary>
        /// 工单标题
        /// </summary>
        public string? OrderTitle { get; set; }

        /// <summary>
        /// 发布人
        /// </summary>
        public string? PubMan { get; set; }

        /// <summary>
        /// 发布范围
        /// </summary>
        public EPublicState? PubRange { get; set; }

        /// <summary>
        /// 受理类型
        /// </summary>
        public List<string>? AcceptTypes { get; set; } = new();

        /// <summary>
        /// 热点分类
        /// </summary>
        public List<string>? HotspotIds { get; set; } = new();

        /// <summary>
        /// 受理时间(工单创建时间)
        /// </summary>
        public DateTime? CreationTimeStart { get; set; }
        public DateTime? CreationTimeEnd { get; set; }

        /// <summary>
        /// 归档时间
        /// </summary>
        public DateTime? FiledTimeStart { get; set; }
        public DateTime? FiledTimeEnd { get; set; }

        /// <summary>
        /// 归档方式
        /// </summary>
        public FiledType? FiledType { get; set; }

        /// <summary>
        /// 是否会签
        /// </summary>
        public bool? IsCountersign { get; set; }

    }

    public enum FiledType 
    {
        [Description("中心归档")]
        CenterFiled = 10,

        [Description("部门归档")]
        OrgFiled = 20,
    }

    public record QueryOrderRedoRecordDto: PagedKeywordRequest
    {

    }


    public record CanDelayOrderListDto : PagedKeywordRequest
    {

    }

    public record CancelDelayDto
    {
        public string Id { get; set; }
    }

    public record ApplyDelayDto
    {
        public string OrderId { get; set; }

        public int DelayNum { get; set; }

        public ETimeType DelayUnit { get; set; }

        public string DelayReason { get; set; }

        public List<FileDto> Files { get; set; }
    }

    public record OrderDelayWebDto
    {

    }

    public class PublishOrderDelayDto
    {
        public string Id { get; set; }

        /// <summary>
        /// 工单编号（冗余）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 工单ID
        /// </summary>
        public string OrderId { get; set; }

        public OrderDto? Order { get; set; }

        /// <summary>
        /// 延期申请时间
        /// </summary>
        public DateTime ApplyDelayTime { get; set; }

        /// <summary>
        /// 申请部门Code
        /// </summary>
        public string ApplyOrgCode { get; set; }

        /// <summary>
        /// 申请部门名称
        /// </summary>
        public string ApplyOrgName { get; set; }

        /// <summary>
        /// 申请人
        /// </summary>
        public string EmployeeId { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public string? EmployeeName { get; set; }

        /// <summary>
        /// 延期申请时限
        /// </summary>
        public int DelayNum { get; set; }

        /// <summary>
        /// 延期申请单位
        /// </summary>
        public ETimeType DelayUnit { get; set; }

        /// <summary>
        /// 延期申请理由
        /// </summary>
        public string DelayReason { get; set; }

        /// <summary>
        /// 申请前期满时间
        /// </summary>
        public DateTime? BeforeDelay { get; set; }

        /// <summary>
        /// 申请后期满时间
        /// </summary>
        public DateTime? AfterDelay { get; set; }

        /// <summary>
        /// 审批状态
        /// </summary>
        public EDelayState DelayState { get; set; }

        /// <summary>
        /// 是否省延期
        /// </summary>
        public bool IsProDelay { get; set; }

        public List<FileJson> FileJson { get; set; }
    }

    public record OrderDelayDto
    {
        /// <summary>
        /// 工单编号（冗余）
        /// </summary>
        public string No { get; set; }

        /// <summary>
        /// 工单ID
        /// </summary>
        public string OrderId { get; set; }

        public OrderDto? Order { get; set; }

        /// <summary>
        /// 延期申请时间
        /// </summary>
        public DateTime ApplyDelayTime { get; set; }

        /// <summary>
        /// 申请部门Code
        /// </summary>
        public string ApplyOrgCode { get; set; }

        /// <summary>
        /// 申请部门名称
        /// </summary>
        public string ApplyOrgName { get; set; }

        /// <summary>
        /// 申请人
        /// </summary>
        public string EmployeeId { get; set; }

        /// <summary>
        /// 
        /// </summary>
        public string? EmployeeName { get; set; }

        /// <summary>
        /// 延期申请时限
        /// </summary>
        public int DelayNum { get; set; }

        /// <summary>
        /// 延期申请单位
        /// </summary>
        public ETimeType DelayUnit { get; set; }

        public string DelayUnitText => DelayUnit.GetDescription();

        /// <summary>
        /// 延期申请理由
        /// </summary>
        public string DelayReason { get; set; }

        /// <summary>
        /// 申请前期满时间
        /// </summary>
        public DateTime? BeforeDelay { get; set; }

        /// <summary>
        /// 申请后期满时间
        /// </summary>
        public DateTime? AfterDelay { get; set; }

        /// <summary>
        /// 审批状态
        /// </summary>
        public EDelayState DelayState { get; set; }

        public string DelayStateText => DelayState.GetDescription();

        /// <summary>
        /// 是否省延期
        /// </summary>
        public bool IsProDelay { get; set; }

        /// <summary>
        /// 流程ID
        /// </summary>
        public string? WorkflowId { get; set; }

        public DateTime CreationTime { get; set; }

        public string Id { get; set; }

        public string? CreatorId { get; set; }

        public string? CreatorName { get; set; }

        public string? CreatorOrgId { get; set; }

        public string? CreatorOrgName { get; set; }

        public int CreatorOrgLevel { get; set; }

        /// <summary>
        /// 一级部门Id
        /// </summary>
        public string? AreaId { get; set; }

        /// <summary>
        /// 当前办理节点
        /// </summary>
        public string? CurrentStepName { get; set; }
        /// <summary>
        /// 当前办理人
        /// </summary>
        public string? ActualHandlerName { get; set; }

        /// <summary>
        /// 是否可办理
        /// </summary>
        public bool IsCanHandle { get; set; }

        public List<FileDto> Files { get; set; }

        public List<FileJson> FileJson { get; set; }

        /// <summary>
        /// 办理 true  审批 false 
        /// </summary>
        public bool Handle { get; set; }
	}


    public record DelayCalcEndTimeDto
    {
        public DateTime BeginTime { get; set; }

        public int DelayNum { get; set; }

        public ETimeType DelayUnit { get; set; }
    }

    public record DelayListDto : PagedKeywordRequest
    {
        /// <summary>
        /// 是否已办 true:已办 false:待办
        /// </summary>
        public bool? IsApply { get; set; }

        public EDelayState? DelayState { get; set; }
    }

    public record ApplySuperviseDto
	{
	    public string OrderId { get; set; }
    }

    public record SuperviseListDto : PagedKeywordRequest
    {
	    /// <summary>
	    /// 督办回复状态
	    /// </summary>
	    public int? SuperviseState { get; set; }

	    public DateTime? CreationTimeStart { get; set; }
	    public DateTime? CreationTimeEnd { get; set; }
	}

    public record ScreenListDto : PagedKeywordRequest
    {
		/// <summary>
		/// 甄别申请状态
		/// </summary>
		public EScreenStatus? Status { get; set; }

		public DateTime? CreationTimeStart { get; set; }
	    public DateTime? CreationTimeEnd { get; set; }

        public string? OrderId { get; set; }

        /// <summary>
        /// 甄别查询来源  1 甄别待办  2 甄别列表
        /// </summary>
        public int source { get; set; }

	}

	public record MayScreenListDto : PagedKeywordRequest
    {
	    /// <summary>
	    /// 部门编号
	    /// </summary>
	    public string VisitOrgCode { get; set; }

	    public DateTime? CreationTimeStart { get; set; }
	    public DateTime? CreationTimeEnd { get; set; }

        public bool? IsHomePage { get; set; }
    }

	public record UrgeListDto : PagedKeywordRequest
    {
	    /// <summary>
	    /// 回复状态
	    /// </summary>
	    public int? UrgeState { get; set; }
	    public DateTime? CreationTimeStart { get; set; }
	    public DateTime? CreationTimeEnd { get; set; }
    }

    public class QueryRepeatableEventDto {
		public string? HotspotSpliceName { get; set; }

        public string? Address { get; set; }
	}


    public enum EPublicState
    {
        /// <summary>
        /// 全部
        /// </summary>
        [Description("全部")]
        All = 0,
        /// <summary>
        /// 公开
        /// </summary>
        [Description("公开")]
        Pub = 1,
        /// <summary>
        /// 不公开
        /// </summary>
        [Description("不公开")]
        NoPub = 2,
    }


    public enum EPubState
    {
        /// <summary>
        /// 全部
        /// </summary>
        [Description("全部")]
        All = 0,
        /// <summary>
        /// 已发布
        /// </summary>
        [Description("已发布")]
        Pub = 1,
        /// <summary>
        /// 未发布
        /// </summary>
        [Description("未发布")]
        NoPub = 2
    }

}
