﻿
using Hotline.Share.Dtos.FlowEngine.Workflow;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.CallCenter;
using Hotline.Share.Requests;
using XF.Utility.EnumExtensions;

namespace Hotline.Share.Dtos.TrCallCenter
{
    #region 分机
    public class TrTelDto
    {
        public string Id { get; set; }

        public string Name { get; set; }

        public string TelNo { get; set; }

        public string Description { get; set; }

        public string QueueId { get; set; }
    }


    public class TrTelStateDto
    {
        /// <summary>
        /// ID
        /// </summary>
        public string? Id { get; set; }

        /// <summary>
        /// 分机号
        /// </summary>
        public string? TelNo { get; set; }
        /// <summary>
        /// 通道流水
        /// </summary>
        public string? ChannelUUid { get; set; }
        /// <summary>
        /// 鉴权用户名
        /// </summary>
        public string? TelName { get; set; }
        /// <summary>
        /// 分机状态机类型
        /// </summary>
        public string? Type { get; set; }
        /// <summary>
        /// 权重
        /// </summary>
        public string? Weight { get; set; }

        /// <summary>
        /// 所在呼叫组
        /// </summary>
        public string? Queue { get; set; }

        /// <summary>
        /// 分机状态
        /// </summary>
        public string? State { get; set; }

        /// <summary>
        /// 前一状态
        /// </summary>
        public string? OldState { get; set; }

        /// <summary>
        /// 注册设备
        /// </summary>
        public string? Device { get; set; }

        /// <summary>
        /// 注册IP
        /// </summary>
        public string? SipIp { get; set; }

        /// <summary>
        /// 二级状态
        /// </summary>
        public string? PrivateData { get; set; }

        /// <summary>
        /// 是否注册
        /// </summary>
        public string? SipState { get; set; }

        /// <summary>
        /// 签入时间
        /// </summary>
        public string? CreatedAt { get; set; }

        /// <summary>
        /// 状态变更时间
        /// </summary>
        public string? UpdatedAt { get; set; }

        /// <summary>
        /// 呼叫方向
        /// </summary>
        public string? CallDirection { get; set; }

        /// <summary>
        /// 呼叫号码
        /// </summary>
        public string? OtherNumber { get; set; }

        /// <summary>
        /// 使用线路
        /// </summary>
        public string? GateWay { get; set; }

        /// <summary>
        /// 接听时间
        /// </summary>
        public string? AnsweredAt { get; set; }
    }



    #endregion

    #region 黑白名单
    public class TrAddBlacklistDto
    {
        public string Phone { get; set; }

        public int SpecialFlag { get; set; }
    }

    public class TrDelBlacklistDto
    {
        public string Phone { get; set; }

        public int? SpecialFlag { get; set; }
    }

    public class TrQueryBlacklistDto
    {
        public string? Phone { get; set; }

        /// <summary>
        /// 1:白名单;2:呼入黑名单;3:呼出黑名单;4:呼入呼出黑名单;
        /// </summary>
        public int? SpecialFlag { get; set; }
    }

    public class TrQueryBlacklistResponseDto
    {
        public string Id { get; set; }

        public string UserId { get; set; }

        public DateTime CreationTime { get; set; }

        public string Phone { get; set; }

        public int SpecialFlag { get; set; }

        public int? Priority { get; set; }
    }
    #endregion

    #region 登录

    public class TrOnDutyDto
    {
        public string? TelNo { get; set; }
    }

    public class TrOnDutyResponseDto
    {
        public string? TelNo { get; set; }

        public string? TelPwd { get; set; }

        public string? Description { get; set; }
        public string? QueueId { get; set; }

        public DateTime StartTime { get; set; }
    }


    #endregion

    #region 小休

    public class TrRestDto
    {
        public string Reason { get; set; }
    }

    #endregion

    #region 通话记录（对外）

    public class ReceiveCallRecordDto
    {
        /// <summary>
        /// IPPBX用户名
        /// </summary>
        public string user { get; set; }

        /// <summary>
        /// 可直接访问的通话录音地址
        /// </summary>
        public string? recording_file_url { get; set; }

        /// <summary>
        /// 呼叫方向：呼入 in / 呼出 out
        /// </summary>
        public string call_direction { get; set; }

        /// <summary>
        /// 挂断方：主叫挂断 caller / 被叫挂断 callee / 其他未知情况
        /// </summary>
        public string hangup_side { get; set; }

        /// <summary>
        /// 转接分机号
        /// </summary>
        public string? agent_transfer_number { get; set; }

        /// <summary>
        /// 坐席侧通话流水，无坐席参与则为手机侧通话流水
        /// </summary>
        public string call_accept { get; set; }

        /// <summary>
        /// 主叫号码
        /// </summary>
        public string caller_id_name { get; set; }

        /// <summary>
        /// 主叫号码
        /// </summary>
        public string caller_id_number { get; set; }

        /// <summary>
        /// 被叫号码
        /// </summary>
        public string destination_number { get; set; }

        /// <summary>
        /// 通话录音绝对路径
        /// </summary>
        public string? recording_file_name { get; set; }

        /// <summary>
        /// 通话创建时间
        /// </summary>
        public string created_time { get; set; }

        /// <summary>
        /// 接通时间
        /// </summary>
        public string? answered_time { get; set; }

        /// <summary>
        /// 挂机时间
        /// </summary>
        public string over_time { get; set; }

        /// <summary>
        /// 线路号
        /// </summary>
        public string gateway { get; set; }

        /// <summary>
        /// 呼出时自定义的内容
        /// </summary>
        public string? other_str { get; set; }

        /// <summary>
        /// 手机侧通话流水，无坐席参与则为空
        /// </summary>
        public string? other_accept { get; set; }

        /// <summary>
        /// 状态码，语音信箱：999/其他：16
        /// </summary>
        public string status { get; set; }

        /// <summary>
        /// IVR开始时间
        /// </summary>
        public string? beginIvrTime { get; set; }

        /// <summary>
        /// IVR结束时间
        /// </summary>
        public string? endIvrTime { get; set; }

        /// <summary>
        /// 开始等待时间
        /// </summary>
        public string? beginQueueTime { get; set; }

        /// <summary>
        /// 结束等待时间
        /// </summary>
        public string? endQueueTime { get; set; }

        /// <summary>
        /// 开始振铃时间
        /// </summary>
        public string? beginRingTime { get; set; }

        /// <summary>
        /// 结束振铃时间
        /// </summary>
        public string? endRingTime { get; set; }

        /// <summary>
        /// 队列ID
        /// </summary>
        public string? ola_queue { get; set; }

        /// <summary>
        /// 批量外呼流水
        /// </summary>
        public string? batch_accept { get; set; }

        /// <summary>
        /// IVR按键信息
        /// </summary>
        public string? ivr_dtmf { get; set; }

        /// <summary>
        /// IVR按键对应动作，需解码
        /// </summary>
        public string? dtmf_type { get; set; }
    }


    public class ReceiveCallEvaluateDto
    {
        public string dtmf { get; set; }

        public string call_accept { get; set; }
        public int type { get; set; }
    }

    public class PublishCallRecrodDto
    {
        public OrderDto? Order { get; set; }

        /// <summary>
        /// 办理节点信息
        /// </summary>
        public WorkflowTraceDto WorkflowTrace { get; set; }

        public TrCallDto? TrCallRecordDto { get; set; }
    }

    #endregion

    #region 通话记录（对内）

    public record class GetCallListDto : PagedRequest
    {
        /// <summary>
        /// 主叫
        /// </summary>
        public string? CPN { get; set; }
        /// <summary>
        /// 被叫
        /// </summary>
        public string? CDPN { get; set; }

        /// <summary>
        /// 分机号
        /// </summary>
        public string? TelNo { get; set; }

        /// <summary>
        /// 用户名
        /// </summary>
        public string? UserName { get; set; }

        /// <summary>
        /// 电话方向
        /// </summary>
        public ECallDirection? CallDirection { get; set; }

        /// <summary>
        /// 通话状态
        /// </summary>
        public EOnState? OnState { get; set; }

        /// <summary>
        /// 挂断方
        /// </summary>
        public EEndBy? EndBy { get; set; }

        /// <summary>
        /// IVR开始时间
        /// </summary>
        public DateTime? BeginIvrTime { get; set; }
        /// <summary>
        /// IVR结束时间
        /// </summary>
        public DateTime? EndIvrTime { get; set; }

        /// <summary>
        /// 开始等待时间(队列开始时间)
        /// </summary>
        public DateTime? BeginQueueTime { get; set; }
        /// <summary>
        /// 结束等待时间（队列结束时间）
        /// </summary>
        public DateTime? EndQueueTime { get; set; }

        /// <summary>
        /// 接通开始时间
        /// </summary>
        public DateTime? AnsweredTime { get; set; }
        /// <summary>
        /// 接通结束时间
        /// </summary>
        public DateTime? OverTime { get; set; }

        /// <summary>
        /// 工单编号
        /// </summary>
        public string? OrderNo { get; set; }

        /// <summary>
        /// 工单标题
        /// </summary>
        public string? Title { get; set; }

        /// <summary>
        /// 中继号
        /// </summary>
        public string? Gateway { get; set; }
        ///// <summary>
        ///// 开始时间
        ///// </summary>
        //public DateTime? StartTime { get; set; }

        ///// <summary>
        ///// 结束时间
        ///// </summary>
        //public DateTime? EndTime { get; set; }

    }


    public class TrCallDto
    {
        /// <summary>
        /// 可直接访问的通话录音地址
        /// </summary>
        public string? RecordingFileUrl { get; set; }
        /// <summary>
        /// 呼叫方向
        /// </summary>
        public ECallDirection CallDirection { get; set; }

        /// <summary>
        /// 呼叫方向
        /// </summary>
        public string CallDirectionText => CallDirection.GetDescription();

        /// <summary>
        /// 挂断方
        /// </summary>
        public EEndBy? EndBy { get; set; }

        /// <summary>
        /// 挂断方
        /// </summary>
        public string EndByText => EndBy.GetDescription();


        /// <summary>
        /// 坐席侧通话流水，无坐席参与则为手机侧通话流水
        /// </summary>
        public string CallAccept { get; set; }
        /// <summary>
        /// 主叫
        /// </summary>
        public string CPN { get; set; }
        /// <summary>
        /// 主叫名称
        /// </summary>
        public string CPNName { get; set; }
        /// <summary>
        /// 被叫
        /// </summary>
        public string CDPN { get; set; }
        /// <summary>
        /// 分机号
        /// </summary>
        public string TelNo { get; set; }
        /// <summary>
        /// 用户ID
        /// </summary>
        public string UserId { get; set; }
        /// <summary>
        /// 用户名
        /// </summary>
        public string UserName { get; set; }
        /// <summary>
        /// 通话录音绝对路径
        /// </summary>
        public string? RecordingFileName { get; set; }
        /// <summary>
        /// 通话创建时间
        /// </summary>
        public DateTime CreatedTime { get; set; }
        /// <summary>
        /// 接通时间
        /// </summary>
        public DateTime? AnsweredTime { get; set; }
        /// <summary>
        /// 挂机时间
        /// </summary>
        public DateTime OverTime { get; set; }
        /// <summary>
        /// 线路号
        /// </summary>
        public string Gateway { get; set; }
        /// <summary>
        /// 呼出时自定义的内容
        /// </summary>
        public string? OtherStr { get; set; }
        /// <summary>
        /// 手机侧通话流水，无坐席参与则为空
        /// </summary>
        public string? OtherAccept { get; set; }
        /// <summary>
        /// 状态码，语音信箱：999/其他：16
        /// </summary>
        public string Status { get; set; }
        /// <summary>
        /// IVR开始时间
        /// </summary>
        public DateTime? BeginIvrTime { get; set; }
        /// <summary>
        /// IVR结束时间
        /// </summary>
        public DateTime? EndIvrTime { get; set; }
        /// <summary>
        /// 开始等待时间
        /// </summary>
        public DateTime? BeginQueueTime { get; set; }
        /// <summary>
        /// 结束等待时间
        /// </summary>
        public DateTime? EndQueueTime { get; set; }
        /// <summary>
        /// 开始振铃时间
        /// </summary>
        public DateTime? BeginRingTime { get; set; }
        /// <summary>
        /// 结束振铃时间
        /// </summary>
        public DateTime? EndRingTimg { get; set; }
        /// <summary>
        /// 队列ID
        /// </summary>
        public string? OlaQueue { get; set; }
        /// <summary>
        /// 批量外呼流水
        /// </summary>
        public string? BatchAccept { get; set; }
        /// <summary>
        /// IVR按键信息
        /// </summary>
        public string? IvrDtmf { get; set; }
        /// <summary>
        /// IVR按键对应动作，需解码
        /// </summary>
        public string? DtmfType { get; set; }

        #region 自定义字段
        /// <summary>
        /// 通话时长（挂机时间-接通时间）
        /// </summary>
        public int Duration { get; set; }

        /// <summary>
        /// 振铃时长（振铃结束时间-振铃开始时间）
        /// </summary>
        public int RingTimes { get; set; }

        /// <summary>
        /// 排队时长（排队结束时间-排队开始时间）
        /// </summary>
        public int QueueTims { get; set; }

        /// <summary>
        /// 通话状态
        /// </summary>
        public EOnState OnState { get; set; }

        /// <summary>
        /// 通话状态
        /// </summary>
        public string OnStateText => OnState.GetDescription();


        public ECallOrderType? CallOrderType { get; set; }

        /// <summary>
        /// 外部ID
        /// </summary>
        public string? ExternalId { get; set; }

        /// <summary>
        /// 工单
        /// </summary>
        public OrderDto Order { get; set; }

        #endregion
    }


    #endregion

    #region 关联

    public class LinkCallRecordDto
    {
        public bool IsOrder { get; set; }

        public string Id { get; set; }

        public string CallId { get; set; }
    }

    #endregion
}
